<?php
namespace App\Util;

use App\Util\Command;
use App\Exception;

class Printing {

    protected $printerProperties;
    protected $command;
    protected $urlRequest;

    const METHOD_GET  = 'GET';
    const METHOD_POST = 'POST';

    const STATUS_CODE_OK        = 200;
    const STATUS_CODE_NOT_FOUND = 404;
    const CONNECTTIMEOUT = 4;
    const TIMEOUT = 18;
    const HEADER_REQUEST = array('Content-Type: application/json'); 

    public function __construct($curl, Command $command) {
        $this->command = $command;
        $this->logPath = $this->logPath();
    }

    private function logPath(){
        global $CLIENT_NAME;
        return $CLIENT_NAME['logPath'];
    }
    

    public function setPrinterProperties(array $printerProperties) {
        $this->printerProperties = $printerProperties;

        return $this;
    }

    public function setUrl($url) {
    	$this->urlRequest = $url;
    }

    public function sendPrint() {
        try {
            $url = $this->urlRequest;
            $result = $this->request($url, self::METHOD_POST ,'/print', array(
                'printerInfo' => $this->printerProperties,
                'commands' => $this->command->getCommands()
            ));

            $resultJson = json_decode($result, true);

            return $resultJson;
        }catch(\Exception $e) {
            throw new \Exception('Endereço do Periféricos inválido ou se encontra desligado.');
        }
    } 

    public function testPrint() {
        try {
            $url = $this->urlRequest;
            $result = $this->request($url, self::METHOD_POST, '/test', array(
                'printerInfo' => $this->printerProperties
            ));

            $resultJson = json_decode($result, true);

            if ($resultJson['error']) {
                throw new \Exception($resultJson['message']);
            } else if ($resultJson == null) {
                throw new \Exception($result);
            }

            return $resultJson;
        }catch(\Exception $e) {
            throw new \Exception('Endereço do Periféricos inválido ou se encontra desligado.');
        }
    }

    private function request($url, $method, $route, $body = null){
       
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL             => $url . $route,
            CURLOPT_RETURNTRANSFER  => true,
            CURLOPT_ENCODING        => '',
            CURLOPT_CONNECTTIMEOUT  => self::CONNECTTIMEOUT,
            CURLOPT_TIMEOUT         => self::TIMEOUT,
            CURLOPT_FOLLOWLOCATION  => true,
            CURLOPT_SSL_VERIFYPEER  => false,
            CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST   => $method,
            CURLOPT_POSTFIELDS      => json_encode($body),
            CURLOPT_HTTPHEADER      => self::HEADER_REQUEST,
        ]);
        
        $curlResponse = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "REQ-IMP" . date("H-i-s") . ".txt";

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => ['Content-Type: application/json'],
                'url'       => $url,
                'method'    => 'POST',
                'params'    => json_encode($body),
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        curl_close($curl);

        if (($httpCode != 200) && ($httpCode != 201))
            $this->writeLog("execution", "", "Erro de Impressao de pedido (httpCode: $httpCode) \r\n");

        return $curlResponse;
    }

    public function writeLog($type, $fileName, $content)
    {
        $basePath     = $this->logPath;

        if ($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if ($type == 'order_pending') {
            $folder      = $basePath . "/pending/";
            $currentFile = "$folder/$fileName";
            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            if ((file_exists($currentFile)))
                return true;
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if ($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if ($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";

            $currentFile = "$folder/$fileName";

            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if ($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

        if ($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";

            $currentFile = "$folder" . date("d") . ".txt";

            if (!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }
    }

    public function formatNetworkError($networkData)
    {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response
        ];

        $data = str_replace(array_keys($templateParams), array_values($templateParams), $template);
        return str_replace("\n", "\r\n", $data);
    }

}